@@ -49,7 +49,7 @@ def get_exon(args):
                 exon = Seq(exon)
                 if t.strand == '-':
                     exon = exon.reverse_complement()
-                exonRecord = SeqRecord(exon,id=args.transcript,
+                exonRecord = SeqRecord(exon,id=t.id,
                              description='strand %s exon %d start %d end %d length=%d'%(t.strand,
                                         exon_index, e.start, e.end, len(exon)))
                 whole_exons.append(exonRecord)
