@@ -291,6 +291,10 @@ parser_uORF.add_argument('-t', '--output_format', choices=['csv','fasta','gff'],
                         help='output format ')
 parser_uORF.add_argument('-o', '--output', type=str, 
                          help='output file path')
+parser_uORF.add_argument('-m', '--schematic_without_intron', type=str,
+                         help='schematic figure file for uORF, CDS and transcript without intron')
+parser_uORF.add_argument('-n', '--schematic_with_intron', type=str,
+                         help='schematic figure file for uORF, CDS and transcript with intron')
 parser_uORF.add_argument('-s', '--style', choices=['GFF','GTF'], 
                          help = 'GTF database or GFF database')
 parser_uORF.set_defaults(func=uORF)
@@ -327,8 +331,10 @@ parser_dORF.add_argument('-t', '--output_format', choices=['csv','fasta','gff'],
                         help='output format ')
 parser_dORF.add_argument('-o', '--output', type=str, 
                          help='output file path')
-parser_dORF.add_argument('-p', '--print', action="store_true", 
-                         help='output to stdout')
+parser_dORF.add_argument('-m', '--schematic_without_intron', type=str,
+                         help='schematic figure file for dORF, CDS and transcript without intron')
+parser_dORF.add_argument('-n', '--schematic_with_intron', type=str,
+                         help='schematic figure file for dORF, CDS and transcript with intron')
 parser_dORF.add_argument('-s', '--style', choices=['GFF','GTF'], 
                          help = 'GTF database or GFF database')
 parser_dORF.set_defaults(func=dORF)
